#!/usr/bin/env python
# -*- coding: utf-8 -*-
#----------------------
# Name: coercion.py
# Python Script
# Author: Raymond Wagner
# Purpose
#    this python script is a proof of concept, for interfacing with the tv show
#    recommendation system provided by Rolisoft (http://lab.rolisoft.net/tv)
#    and provide the user with automatically recorded shows, recommended to the
#    user based off their viewing habits
# --------------------
__title__  = "Coercion"
__author__ = "Raymond Wagner"
__version__= "v0.1"

import urllib
import json
import pickle
import os
from uuid import uuid4
from time import time
from optparse import OptionParser
from MythTV import MythDB, schemaUpdate, DBDataWrite, Record

@schemaUpdate
def run_schema_update(version):
    if version == -1:
        return 'coercion.DBSchemaVer'
    if version is None:
        return ((("""CREATE TABLE IF NOT EXISTS coerced
                        (title VARCHAR(128) NOT NULL,
                         score INT UNSIGNED NOT NULL DEFAULT 0,
                         type TINYINT UNSIGNED NOT NULL DEFAULT 0,
                         recordid INT UNSIGNED)""",()),),
                '1000')
    if version == '1000':
        return ((("""INSERT INTO settings (value, data)
                             VALUES('coercion.UUID',%s)""", str(uuid4())),),
                '1001')
    if version >= '1001':
        raise StopIteration

class SuggestedRec( DBDataWrite ):
    _table = 'coerced'
    _where = 'title=%s'
    _setwheredat = 'self.title,'
    _defaults = {}

    REC = 0
    ACTIVEREC = 1
    BLACKLIST = 2
    EXISTING = 3
    IGNORED = 4

    def __init__(self, data=None, db=None, raw=None):
        DBDataWrite.__init__(self, (data,), db, raw)
        if data is not None:
            db = MythDB(db=self._db)
            recs = db.searchRecord(title=self.title)
            if recs is not None:
                self.recordid = recs[0].recordid

    def record(self, priority):
        if self.recordid is not None:
            return
        rec = Record(db=self._db)
        rec.title = self.title
        rec.type = Record.kAllRecord
        rec.recpriority = priority
        self.recordid = rec.create().recordid
        print 'adding recording rule for %s at priority %d' % (self.title, priority)
        #self.recordid = 10000
        self.type = self.ACTIVEREC
        self.update()

    def unrecord(self):
        rec = Record(self.recordid, self._db)
        rec.delete()
        print 'deleting recording rule for %s' % self.title
        self.type = self.REC
        self.recordid = None
        self.update()

    def blacklist(self, blacklist=True):
        if self.type not in (0,1,2):
            return
        if blacklist:
            print 'moving %s to blacklist' % self.title
            self.type = self.BLACKLIST
        else:
            print 'removing %s from blacklist' % self.title
            self.type = self.REC
        self.update()
        return

    def delete(self):
        print 'deleting %s, type %d' % (self.title, self.type)
        if self.type == self.ACTIVEREC:
            self.unrecord()
        DBDataWrite.delete(self)

def collect_recordings(db):
    return [r.title for r in db.searchRecorded(category_type='Series')]+\
           [r.title for r in db.searchRecorded(notsubtitle='')] 

def collect_record_rules(db):
    return [r.title for r in db.searchRecord(notsubtitle='', custom=(('recpriority>%s', db.settings.NULL.get('coercion.base_priority',-10)),))]

def collect_old_recorded(db):
    return []

def populate_existing(titlelist, db):
    # get an existing list
    recs = [r.title for r in SuggestedRec(db=db).getAllEntries() if r.type in (3,4)]
    for title in list(titlelist):
        if title in recs:
            # matches existing entry
            #print 'matched existing entry %s' % title
            recs.remove(title)
        else:
            # add new entry
            #print 'adding new entry %s' % title
            rec = SuggestedRec(db=db)
            rec.title = title
            rec.type = rec.EXISTING
            rec.create()
    for title in recs:
        # remove old entries
        #print 'deleting old entry %s' % title
        SuggestedRec(title, db=db).delete()
    return [r.title for r in SuggestedRec(db=db).getAllEntries() if r.type == 3]

def populate_recommended(titlelist, db):
    # get an existing list
    recs = dict([(r.title, r) for r in SuggestedRec(db=db).getAllEntries() if r.type in (0,1,2)])
    for title in list(titlelist):
        if title[0] in recs:
            # matches existing entry
            #print 'matched existing entry %s' % title[0]
            rec = recs[title[0]]
            rec.score = title[1]
            rec.update()
            del recs[title[0]]
        else:
            # add new entry
            #print 'adding new entry %s' % title[0]
            rec = SuggestedRec(db=db)
            rec.title = title[0]
            rec.type = rec.REC
            rec.create()
    for rec in recs.values():
        # remove old recs
        if rec.type in (0,1):
            #print 'deleting old entry %s' % rec.title
            rec.delete()

def populate_rules(db):
    recs = [r for r in SuggestedRec(db=db).getAllEntries() if r.type in (0,1)]
    recs.sort(key=lambda rec: rec.score, reverse=True)
    cut = min(int(db.settings.NULL.get('coercion.count',8)), len(recs))
    for i in range(cut):
        if recs[i].type == 0:
            recs[i].record(int(db.settings.NULL.get('coercion.base_priority',-10))-i)
    for i in range(cut, len(recs)):
        if recs[i].type == 1:
            recs[i].unrecord()

def fetch_rolisoft(showlist, uuid):
    cachefile = '/tmp/coercion.rolisoft.pickle'
    try:
        cache = pickle.load(open(cachefile,'r'))
    except:
        cache = {}
    for key in list(cache.keys()):
        if cache[key]['time'] < time() - 7*24*60*60:
            del cache[key]

    cachekey = hash(tuple(showlist))
    if cachekey not in cache:
        #load translation file
        transfile = os.path.expanduser('~/.mythtv/rolisoft.conf')
        trans = {}
        if os.access(transfile, os.F_OK):
            for line in open(transfile):
                if line[0] == '#':
                    continue
                line = line.strip().split('=',1)
                trans[line[0]] = line[1]
        # translate shows
        for i in range(len(showlist)):
            if showlist[i] in trans:
                showlist[i] = trans[showlist[i]]
        # flip dictionary
        trans = dict(zip(*reversed(zip(*trans.items()))))

        apikey='pf7PczNoCGsY8AvglfV4gmjMt'
        url = 'http://lab.rolisoft.net/tv/api.php?key=%s&output=json&uid=%s&%s' % \
                (apikey, uuid, '&'.join('show[]=%s' % urllib.quote(show) for show in showlist))
        #print url
        fd = urllib.urlopen(url)
        page = fd.read()
        fd.close()
        data = json.loads(page)
        #print data

        # translate response
        for d in data:
            if d['name'] in trans:
                d['name'] = trans[d['name']]

        cache[cachekey] = {'time':time(), 'data':data}
        pickle.dump(cache, open(cachefile,'w'))
    else:
        data = cache[cachekey]['data']
    return [(d['name'],d['score']) for d in data]



def main():
#    parser = OptionParser(usage="usage: %prog [options]")

#    parser.add_option("-c", "--count", action="store", type="int", default=2, dest="count",
#            help="Specify number of recommended shows to set to record.")
#    parser.add_option("-p", "--priority", action="store", type="int", default=-10, dest="priority",
#            help="Specify the priority to set to recommended recordings.")
#    parser.add_option("-s", "--simulation", action="store_true", default=False, dest="simulate",
#            help="Simulation (dry run). No rules are added, just printed to the command line.")
#    parser.add_option("-v", "--verbose", action="store_true", default=False, dest="verbose",
#            help="Enable increased verbosity.")

#    opts, args = parser.parse_args()

    db = MythDB()

    run_schema_update(db)

    titlelist = list(set(collect_recordings(db)+\
                         collect_record_rules(db)+\
                         collect_old_recorded(db)))

    titlelist = populate_existing(titlelist, db)
    recommendlist = fetch_rolisoft(titlelist, db.settings.NULL['coercion.UUID'])
    populate_recommended(recommendlist, db)
    populate_rules(db)
    
if __name__ == "__main__":
    main()
